/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdColor.h"

using namespace std;
using namespace FD;
   
class CvfdCreateColor;
   
DECLARE_NODE(CvfdCreateColor)
/*Node
 *
 * @name CvfdCreateColor
 * @category FDOPENCV_ORG:Color
 * @description Create a 3 channels color (ex: RGB, HSV...).
 *
 * @parameter_name CHANNEL1
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description channel1
 *
 * @parameter_name CHANNEL2
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description channel2
 *
 * @parameter_name CHANNEL3
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description channel3
 * 
 * @output_name COLOUR
 * @output_description The element structure of the color
 * @output_type CvfdColor
 *
 END*/
   
   
class CvfdCreateColor : public BufferedNode {
  //Output ID
  int m_colourID;
  //Parameters
  int m_channel1;
  int m_channel2;
  int m_channel3;
      
public:
  CvfdCreateColor(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add outputs
    m_colourID = addOutput("COLOUR");
    //Initialize parameters         
    m_channel1 = dereference_cast<int>(parameters.get("CHANNEL1"));
    m_channel2 = dereference_cast<int>(parameters.get("CHANNEL2"));
    m_channel3 = dereference_cast<int>(parameters.get("CHANNEL3"));
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {   
    //Handle
    CvfdColor* color = new CvfdColor(m_channel1, m_channel2, m_channel3);  
    out[count] = ObjectRef(color);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdCreateColor)
};
